<?php
namespace Webshop\Options\Service;

use Doctrine\ORM\EntityManagerInterface;
use Webshop\Options\Option;
use Webshop\Options\OptionInterface;

/**
 * Class OptionManager
 *
 * @package Webshop\Options\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/webshop-options
 */
final class OptionManager implements OptionManagerInterface
{
    /** @var EntityManagerInterface */
    private $em;

    /** @var OptionInterface[] */
    private $options = [];
    /** @var bool */
    private $autoloadedOptionsLoaded = false;

    /**
     * OptionManager constructor.
     * @param EntityManagerInterface $em
     */
    public function __construct(EntityManagerInterface $em)
    {
        $this->em = $em;
    }

    /**
     * @return EntityManagerInterface
     */
    public function getEntityManager(): EntityManagerInterface
    {
        return $this->em;
    }

    /**
     * Da li postoji postavka s datim imenom?
     *
     * @param string $name
     * @return bool
     */
    public function hasOption(string $name): bool
    {
        return $this->loadOption($name) instanceof OptionInterface;
    }

    /**
     * Stvara novu postavku s datim imenom.
     *
     * @param string $name
     * @param bool $isAutoloaded
     * @return OptionInterface
     */
    public function createOption(string $name, bool $isAutoloaded = false): OptionInterface
    {
        if ($this->hasOption($name)) {
            return $this->getOption($name);
        }

        $option = new Option($name, $isAutoloaded);

        $this->em->persist($option);
        $this->options[$name] = $option;

        return $option;
    }

    /**
     * Vraća postavku s datim imenom.
     *
     * @param string $name
     * @return OptionInterface
     */
    public function getOption(string $name): OptionInterface
    {
        if (!isset($this->options[$name])) {
            return $this->loadOption($name);
        }

        return $this->options[$name];
    }

    /**
     * @param string $name
     */
    public function removeOption(string $name): void
    {
        if ($option = $this->loadOption($name)) {
            $this->em->remove($option);
        }
    }

    /**
     * @param string $name
     * @return OptionInterface
     */
    private function loadOption(string $name): ?OptionInterface
    {
        if (!$this->autoloadedOptionsLoaded) {
            $this->loadAutoloadedOptions();
        }

        if (isset($this->options[$name])) {
            return $this->options[$name];
        }

        /** @var OptionInterface[] $options */
        $options = $this->em->createQueryBuilder()
            ->select('o')
            ->from(Option::class, 'o')
            ->where('o.name = :name')
            ->setParameter('name', $name)
            ->getQuery()
            ->getResult();

        if (empty($options)) {
            return null;
        }

        return $this->options[$name] = $options[0];
    }

    /**
     * Učitava sve postavke koje imaju isAutoloaded true
     */
    private function loadAutoloadedOptions(): void
    {
        if ($this->autoloadedOptionsLoaded) {
            return;
        }

        /** @var OptionInterface[] $options */
        $options = $this->em->createQueryBuilder()
            ->select('o')
            ->from(Option::class, 'o')
            ->where('o.isAutoloaded = :autoload')
            ->setParameter('autoload', 'yes')
            ->getQuery()
            ->getResult();

        foreach ($options as $option) {
            $this->options[$option->getName()] = $option;
        }

        $this->autoloadedOptionsLoaded = true;
    }
}